﻿#region

using System;
using System.Linq;
using CDNSync.Providers.Model;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;

#endregion

namespace CDNSync.MSBuild
{
    public class SyncTask : Task
    {
        private BucketRegion bucketRegion;
        private CDNProvider cdnProvider;
        private bool recursive;
        private bool useFolderNameInKey;

        public SyncTask()
        {
            Files = new ITaskItem[0];
            Folders = new ITaskItem[0];
            GZipExtensions = new ITaskItem[0]; 
        }

        public string Provider { get; set; }

        public string ProviderUserName { get; set; }

        public string ProviderPassword { get; set; }

        public string BucketName { get; set; }

        public string BucketRegion { get; set; }

        public ITaskItem[] Files { get; set; }

        public ITaskItem[] Folders { get; set; }

        public ITaskItem[] GZipExtensions { get; set; }

        public string UseFolderNameInKey { get; set; }

        public string Recursive { get; set; }

        public string FileNamePrefix { get; set; }

        public string FileNameSuffix { get; set; }

        public override bool Execute()
        {
            var request = BuildSyncFilesRequest();

            var synchronizer = new Synchronizer();
            var result = new SyncFilesResult();
            try
            {
                result = synchronizer.SyncFiles(request);

                foreach (var lm in result.LogMessages)
                {
                    LogMessage(lm);
                }
                foreach (var e in result.Errors)
                {
                    LogError(e);
                }
            }
            catch (Exception ex)
            {
                LogError(ex.ToString());
            }


            return result.WasSuccessful;
        }

        private SyncFilesRequest BuildSyncFilesRequest()
        {
            
            bucketRegion = (BucketRegion) Enum.Parse(typeof (BucketRegion), BucketRegion);
            cdnProvider = (CDNProvider) Enum.Parse(typeof (CDNProvider), Provider); 
          
            useFolderNameInKey = !string.IsNullOrEmpty(UseFolderNameInKey) ? UseFolderNameInKey.ToLowerInvariant() == "true" : false;
            recursive = !string.IsNullOrEmpty(Recursive) ? Recursive.ToLowerInvariant() == "true" : false;


            var request = new SyncFilesRequest
                              {
                                  BucketName = BucketName,
                                  Files = Files.Select(f => f.ItemSpec).ToArray(),
                                  Password = ProviderPassword,
                                  UserName = ProviderUserName,
                                  Region = bucketRegion,
                                  Provider = cdnProvider,
                                  UseFolderNameInKey = useFolderNameInKey,
                                  Recursive = recursive,
                                  Folders = Folders.Select(f => f.ItemSpec).ToArray(),
                                  FileNamePrefix = FileNamePrefix,
                                  FileNameSuffix = FileNameSuffix,
                                  GZipExtensions = GZipExtensions.Select(g => g.ItemSpec).ToArray()
                              };

            return request;
        }

        private void LogMessage(string message)
        {
            Log.LogMessage(MessageImportance.Normal, message);
            Console.WriteLine(message);
        }

        private void LogError(string error)
        {
            Log.LogError(error);
            Console.WriteLine(error);
        }
    }
}